/* qdbusdata.h DBUS data transport type
 *
 * Copyright (C) 2007 Kevin Krammer <kevin.krammer@gmx.at>
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 *
 */

#ifndef QDBUSDATA_H
#define QDBUSDATA_H

#include "dbus/qdbusmacros.h"
#include <qglobal.h>

class QCString;
class QDBusDataList;
class QDBusVariant;
class QString;

template<typename T> class QValueList;
template<typename T> class QDBusDataMap;

class QDBUS_EXPORT QDBusData
{
public:
    enum Type
    {
        Invalid = 0,
        Bool,
        Byte,
        Int16,
        UInt16,
        Int32,
        UInt32,
        Int64,
        UInt64,
        Double,
        String,
        List,
        Struct,
        Variant,
        Map
    };

    QDBusData();

    QDBusData(const QDBusData& other);

    ~QDBusData();

    QDBusData& operator=(const QDBusData& other);

    bool operator==(const QDBusData& other) const;
    bool operator!=(const QDBusData& other) const;

    inline bool isValid() const { return type() != QDBusData::Invalid; }

    Type type() const;

    Type keyType() const;

    inline const char* typeName() const { return typeName(type()); }

    static const char* typeName(Type type);

    static QDBusData fromBool(bool value);

    bool toBool(bool* ok = 0) const;

    static QDBusData fromByte(Q_UINT8 value);

    Q_UINT8 toByte(bool* ok = 0) const;

    static QDBusData fromInt16(Q_INT16 value);

    Q_INT16 toInt16(bool* ok = 0) const;

    static QDBusData fromUInt16(Q_UINT16 value);

    Q_UINT16 toUInt16(bool* ok = 0) const;

    static QDBusData fromInt32(Q_INT32 value);

    Q_INT32 toInt32(bool* ok = 0) const;

    static QDBusData fromUInt32(Q_UINT32 value);

    Q_UINT32 toUInt32(bool* ok = 0) const;

    static QDBusData fromInt64(Q_INT64 value);

    Q_INT64 toInt64(bool* ok = 0) const;

    static QDBusData fromUInt64(Q_UINT64 value);

    Q_UINT64 toUInt64(bool* ok = 0) const;

    static QDBusData fromDouble(double value);

    double toDouble(bool* ok = 0) const;

    static QDBusData fromString(const QString& value);

    QString toString(bool* ok = 0) const;

    static QDBusData fromList(const QDBusDataList& list);

    QDBusDataList toList(bool* ok = 0) const;

    static QDBusData fromQValueList(const QValueList<QDBusData>& list);

    QValueList<QDBusData> toQValueList(bool* ok = 0) const;

    static QDBusData fromStruct(const QValueList<QDBusData>& memberList);

    QValueList<QDBusData> toStruct(bool* ok = 0) const;

    static QDBusData fromVariant(const QDBusVariant& value);

    QDBusVariant toVariant(bool* ok = 0) const;

    static QDBusData fromByteKeyMap(const QDBusDataMap<Q_UINT8>& map);

    QDBusDataMap<Q_UINT8> toByteKeyMap(bool* ok = 0) const;

    static QDBusData fromInt16KeyMap(const QDBusDataMap<Q_INT16>& map);

    QDBusDataMap<Q_INT16> toInt16KeyMap(bool* ok = 0) const;

    static QDBusData fromUInt16KeyMap(const QDBusDataMap<Q_UINT16>& map);

    QDBusDataMap<Q_UINT16> toUInt16KeyMap(bool* ok = 0) const;

    static QDBusData fromInt32KeyMap(const QDBusDataMap<Q_INT32>& map);

    QDBusDataMap<Q_INT32> toInt32KeyMap(bool* ok = 0) const;

    static QDBusData fromUInt32KeyMap(const QDBusDataMap<Q_UINT32>& map);

    QDBusDataMap<Q_UINT32> toUInt32KeyMap(bool* ok = 0) const;

    static QDBusData fromInt64KeyMap(const QDBusDataMap<Q_INT64>& map);

    QDBusDataMap<Q_INT64> toInt64KeyMap(bool* ok = 0) const;

    static QDBusData fromUInt64KeyMap(const QDBusDataMap<Q_UINT64>& map);

    QDBusDataMap<Q_UINT64> toUInt64KeyMap(bool* ok = 0) const;

    static QDBusData fromStringKeyMap(const QDBusDataMap<QString>& map);

    QDBusDataMap<QString> toStringKeyMap(bool* ok = 0) const;

    QCString buildDBusSignature() const;

private:
    class Private;
    Private* d;
};

#endif
